@@ -10,7 +10,6 @@ import os
 from .utils.__player__ import play
 from .utils.__downloader__ import download
 from .utils.__cast__ import cast
-from .__decryptor__ import decrypt_stream_url
 from .__version__ import __core__
 
 try:
@@ -18,8 +17,9 @@ try:
 except ImportError:
     import json
 
+import hashlib
 import sys
-from urllib.parse import urljoin, quote
+from urllib.parse import urljoin, quote, urlencode, quote_plus
 import time
 from bs4 import BeautifulSoup
 
@@ -39,6 +39,67 @@ DECODER = "https://dec.eatmynerds.live"
 selected_media = None
 selected_subtitles = []
 
+def urlencode_ordered(params, order):
+    # Encode query params in exact order
+    parts = []
+    for k in order:
+        v = params[k]
+        parts.append(f"{k}={quote_plus(str(v))}")
+    return "&".join(parts)
+
+def decrypt_stream_url(embed_link, api_url):
+    # Step 1: Get challenge info
+    resp = client.get(f"{api_url}/challenge")
+    if resp.status_code != 200:
+        print(f"Failed to fetch challenge ({resp.status_code})")
+        print(resp.text)
+        return embed_link, []
+
+    data = resp.json()
+    payload = data.get("payload")
+    signature = data.get("signature")
+    difficulty = int(data.get("difficulty", 0))
+    if not (payload and signature and difficulty):
+        print("Missing challenge data fields")
+        return embed_link, []
+
+    challenge = payload.split(".")[0]
+
+    prefix = "0" * difficulty
+    nonce = 0
+    while True:
+        text = f"{challenge}{nonce}"
+        hash_digest = hashlib.sha256(text.encode("utf-8")).hexdigest()
+        if hash_digest.startswith(prefix):
+            break
+        nonce += 1
+
+    # Step 3: Construct full URL manually with encoded params
+    query_params = {
+        "url": embed_link,
+        "payload": payload,
+        "signature": signature,
+        "nonce": nonce
+    }
+    ordered_query = urlencode_ordered(query_params, ['url', 'payload', 'signature', 'nonce'])
+    full_url = f"{api_url}?{ordered_query}"
+
+    final_resp = client.get(full_url)
+    if final_resp.status_code != 200:
+        print(f"Failed to get decrypted url from decoder ({final_resp.status_code})")
+        print(final_resp.text)
+        return embed_link, []
+
+    final_data = final_resp.json()
+    video_link = final_data.get("file") or ""
+    subtitles = []
+    if "tracks" in final_data:
+        for track in final_data["tracks"]:
+            if track.get("kind") == "captions" and track.get("file"):
+                subtitles.append(track["file"])
+
+    return video_link, subtitles
+
 def parse_episode_range(episode_input: str):
     """Parse episode input to handle ranges like '5-7' or single episodes like '5'"""
     episode_input = episode_input.strip()
